import {
  BadRequestException,
  Injectable,
  InternalServerErrorException,
} from '@nestjs/common';
import { CrudService } from 'nicot';
import { Feedback } from './entities/feedback.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { Not, Repository } from 'typeorm';
import { ConfigService } from '@nestjs/config';
import { Random, Session } from 'koishi';
import { HttpService } from '@nestjs/axios';
import { lastValueFrom } from 'rxjs';
import { PutSession, UseCommand } from 'koishi-nestjs';

@Injectable()
export class FeedbackService extends CrudService(Feedback) {
  private notifyUrl = this.config.get<string>('FEEDBACK_NOTIFY_URL');
  private notifyToken = this.config.get<string>('FEEDBACK_NOTIFY_TOKEN');

  constructor(
    @InjectRepository(Feedback) repo: Repository<Feedback>,
    private config: ConfigService,
    private http: HttpService,
  ) {
    super(repo);
  }

  async notifyFeedback(feedback: Feedback) {
    if (!this.notifyUrl) {
      return;
    }
    try {
      await lastValueFrom(
        this.http.post(
          this.notifyUrl,
          { content: feedback.toNotifyText() },
          {
            headers: {
              Authorization: `Bearer ${this.notifyToken}`,
            },
          },
        ),
      );
    } catch (e) {
      this.log.error(`Failed to notify feedback: ${e.message}`);
    }
  }

  async createFeedback(session: Session, payload: Partial<Feedback>) {
    if (!payload.content) {
      throw new BadRequestException('');
    }
    if (payload.content.length > 10000) {
      throw new BadRequestException('反馈内容过长。');
    }
    const feedback = new Feedback().fromSession(session);
    Object.assign(feedback, payload);
    this.notifyFeedback(feedback).then();
    try {
      await this.repo.save(feedback);
      return await this.repo.count({
        where: { category: payload.category === '周边' ? '周边' : Not('周边') },
      });
    } catch (e) {
      const id = Random.id(10);
      this.log.error(`Failed to save feedback id: ${e.message}`);
      throw new InternalServerErrorException(
        '反馈遇到问题，请提供下列编号与管理员取得联系: ' + id,
      );
    }
  }

  @UseCommand('investigate', '调研')
  private async investigateCommand() {
    return '欢迎访问MC有奖调研功能。目前暂时无进行中的调研。我们一般每1-2个月都会公布有奖调研，欢迎下次再查询。';
  }

  @UseCommand('feedback', '发送反馈。')
  private async feedbackCommand(@PutSession() session: Session) {
    await session.send(
      '您的反馈种类是？（输入数字回复）\n' +
        '1. BUG或举报玩家不当行为\n' +
        '2. 游戏功能/改善 \n' +
        '3. 游戏体验出问题（例如卡、掉线）\n' +
        '4. 其他反馈',
    );
    let choice = 0;
    while (true) {
      const input = await session.prompt();
      choice = parseInt(input.trim());
      if (choice >= 1 && choice <= 4) {
        break;
      }
      await session.send('请选择一个正确的选项（数字1~4）');
    }
    if (choice == 1) {
      return (
        '举报BUG需要提交对战录像或截图。\n' +
        '举报玩家需提供充足的证据，例如骂人截图。\n' +
        '因此请到 ygobbs.com 发帖举报。 感谢！'
      );
    }
    const category = ['游戏功能/改善', '游戏体验问题', '其他'][choice - 2];
    await session.send('您启动了反馈系统！您的下一段文字会作为反馈发给我们！');
    const content = await session.prompt();
    const count = await this.createFeedback(session, { category, content });
    return `反馈成功！谢谢您的意见！
这是我们在决斗暗网服务号收到的第 ${count} 个反馈！`;
  }

  @UseCommand('feedback.shop', '发送周边反馈。')
  private async shopFeedbackCommand(@PutSession() session: Session) {
    await session.send(
      '感谢进入周边反馈功能！周边对MC来说至关重要，MC需要周边来支持服务器维持。\n' +
        '如果您有任何对MCPro游戏周边的反馈，请留言于此，感谢！\n' +
        '我们会非常关心您对我们周边的意见和看法！\n' +
        '为此，希望您可以尽可能清晰表达您的意见，以便我们更好理解您的需求。',
    );
    const content = await session.prompt();
    const count = await this.createFeedback(session, {
      category: '周边',
      content,
    });
    return `我们收到了您的周边反馈。
这是我们在决斗暗网服务号收到的第 ${count} 个周边反馈！`;
  }
}
